#define STRICT

#include <windows.h>
#include <stdio.h>
#include <math.h>
#include "orbitersdk.h"
#include "mfd.h"
#include "parser.h"
#include "mfdvarhandler.h"

//extern double debug;

void MFDvarhandler::addtolist(MFDvariable *item)
{
	if (firstitem==NULL)
	{
		firstitem=lastitem=item;
		item->next=item;
		item->previous=item;
		return;
	}
	MFDvariable *oldlast=lastitem;
	lastitem=item;
	oldlast->next=item;
	firstitem->previous=item;
	item->previous=oldlast;
	item->next=firstitem;
	return;
}

void MFDvarhandler::removefromlist(MFDvariable *item)
{
	if (item->next==item)
	{
		firstitem=lastitem=NULL;
		return;
	}
	if (firstitem==item)
	{
		firstitem=item->next;
	}
	if (lastitem==item)
	{
		lastitem=item->previous;
	}
	item->previous->next=item->next;
	item->next->previous=item->previous;
	for (int a=0;a<5;a++)
	{
		if (item==currvariable[a]) currvariable[a]=NULL;
	}
}



void MFDvarhandler::saveallvariables(FILEHANDLE scn)
{
	char buffer[80];
	MFDvariable *begin,*pointer;
	if (firstitem!=NULL)
	{
		begin=pointer=firstitem;
		do 
		{
			pointer->getname(buffer);
			oapiWriteLine(scn,buffer);
			pointer->getsaveline(buffer);
			oapiWriteLine(scn,buffer);
			pointer=pointer->next;
		}
		while (pointer!=firstitem);
	}
	strcpy(buffer,"Finvars");
	oapiWriteLine(scn,buffer);
}

bool MFDvarhandler::loadallvariables(FILEHANDLE scn)
{
	PARSER parser;
	char *buffer,*member;
	char namebuffer[20];
	int length,tadjmode;

	MFDvariable *begin,*pointer;
	if (firstitem==NULL) return false;
	begin=pointer=firstitem;
	do
	{
		if (!oapiReadScenario_nextline(scn,buffer)) return false;
		pointer->getname(namebuffer);
		if (strcmp(buffer,namebuffer)==0)
		{
			//Name matches
			if (!oapiReadScenario_nextline(scn,buffer)) return false;//Unexpected end
			parser.parseline(buffer);
			if (!parser.getlineelement(0,&member,&length)) return false;//No required element
			sscanf(member,"%i",&tadjmode);
			pointer->setadjmode(tadjmode);//Set the adjustment mode
			if (!parser.getlineelement(1,&member,&length)) return false;//No required element
			pointer->loadvalue(member);//Set the variable to the saved value
			pointer=pointer->next;//move to next variable
		}
		else
		{
			//No match
			if (strcmp(member,"Finvars")==0) return true;//Somehow reached end of section - return anyway
			//Try to load the same variable again.
			//Routine copes with additional variables, but fails on deletions or renames,or missing first variable
		}
	}
	while (begin!=pointer);
	//Now finished variables, look for the end label
	do
	{
		if (!oapiReadScenario_nextline(scn,buffer)) return false;
	}
	while (strcmp(buffer,"Finvars")!=0);
	return true;
}


MFDvarhandler::MFDvarhandler()
{
	firstitem=lastitem=NULL;
	for (int a=0;a<5;a++)currvariable[a]=NULL;
}


MFDvarhandler::~MFDvarhandler()
{
	MFDvariable *temp=firstitem;
	if (temp==NULL) return;
	do
	{
		temp->ivars=NULL;
		temp=temp->next;
	}
	while (temp!=firstitem);
}




void MFDvarhandler::setnextcurrent(int viewmode)
{
	MFDvariable *begin;
	MFDvariable *curvariable=currvariable[viewmode];
	if (firstitem==NULL)
	{
		curvariable=NULL;
		return;
	}
	if (curvariable==NULL)
	{
		curvariable=firstitem;
		if ((curvariable->viewmode1==viewmode || curvariable->viewmode2==viewmode)&&curvariable->showvariable) return;
	}
	begin=curvariable;
	do
	{
		curvariable=curvariable->next;		
		if ((curvariable->viewmode1==viewmode || curvariable->viewmode2==viewmode)&&curvariable->showvariable)
		{
			break;
		}

	}
	while (curvariable!=begin);
	currvariable[viewmode]=curvariable;
}

void MFDvarhandler::setprevcurrent(int viewmode)
{
	MFDvariable *begin;
	MFDvariable *curvariable=currvariable[viewmode];
	if (firstitem==NULL)
	{
		curvariable=NULL;
		return;
	}
	if (curvariable==NULL)
	{
		curvariable=firstitem;
	}
	begin=curvariable;
	do
	{
		curvariable=curvariable->previous;
		if ((curvariable->viewmode1==viewmode || curvariable->viewmode2==viewmode)&&curvariable->showvariable)
		{
			break;
		}

	}
	while (curvariable!=begin);
	currvariable[viewmode]=curvariable;
}



MFDvariable* MFDvarhandler::getcurrent(int viewmode)
{
	MFDvariable *curvariable=currvariable[viewmode];
	if (curvariable==NULL) curvariable=firstitem;//May still be NULL
	if (curvariable==NULL) return NULL;
	if (curvariable->viewmode1!=viewmode && curvariable->viewmode2!=viewmode)
	{
		setnextcurrent(viewmode);
	}
	if (curvariable->viewmode1==viewmode || curvariable->viewmode2==viewmode)
	{
		currvariable[viewmode]=curvariable;
		return curvariable;
	}
	return NULL;
}





