#define STRICT
#define ORBITER_MODULE
#include <windows.h>
#include <stdio.h>
#include <math.h>
#include "orbitersdk.h"

#include "mfd.h"
#include "graph.h"
#include "intercept.h"
#include "mfdvarhandler.h"
#include "basefunction.h"
#include "viewstate.h"
#include "transx.h"

extern double debug;

TransxMFD::TransxMFD (DWORD w, DWORD h, VESSEL *vessel, UINT mfd)
: MFD (w, h, vessel)

// Initialise TransXMFD
{
	//Check to see if new Transxstate is required
	valid=false;
	viewstate=NULL;
	bool success;
	if (mfd>20) debug=mfd;
	success=viewstate::getviewstate(mfd,&viewstate,this);
	if (viewstate==NULL)//Could not create transxstate
	{
		return;
	}
	viewstate->setmfdactive(true);
	//debug=0;
}

TransxMFD::~TransxMFD()
{
	viewstate->setmfdactive(false);
	if (!viewstate->getrenderviewport())//If we're closing, take the state with you, otherwise leave it.
	{
		delete viewstate;//If the view window's closed, delete the state
	}
}



// Called by Orbiter when a screen update is needed
void TransxMFD::Update (HDC hDC)

{
	Title (hDC, "TransX MFD");
	valid=viewstate->doupdate(hDC,W,H,this);
	if (!valid) return;
	SelectDefaultPen(hDC,1);//Switches to an orbiter pen - ensures that on return, no pens created here
	//are given back to Orbiter in the device context
	int linespacing=H/24;
	
	if (debug!=0)
	{
		char buffer[20];

		int length=sprintf(buffer,"Debug:%g",debug);
		TextOut(hDC,0,linespacing*22,buffer,length);
	}

	MFDvariable *varpointer=viewstate->GetCurrVariable();
	if (varpointer==NULL) return;

	varpointer->show(hDC,W,linespacing);
}

int TransxMFD::getwidth()
{
	return W;
}

int TransxMFD::getheight()
{
	return H;
}



int TransxMFD::MsgProc (UINT msg, UINT mfd, WPARAM wparam, LPARAM lparam)

// Standard message parser for messages passed from Orbiter

{
	switch (msg) {
	case OAPI_MSG_MFD_OPENED:
		return (int) new TransxMFD (LOWORD(wparam), HIWORD(wparam), (VESSEL*)lparam, mfd);
	}
	return 0;
}

char *TransxMFD::ButtonLabel (int bt)

// Routine to pass button label back to Orbiter. Called by Orbiter

{
	char *label[11] = {"HLP","FWD","BCK", "VW","VAR","-VR", "ADJ", "-AJ","++", "--","SW"};
	return (bt < 11 ? label[bt] : 0);
}

int TransxMFD::ButtonMenu (const MFDBUTTONMENU **menu) const

// Routine to pass menu description for buttons back to Orbiter. Called by Orbiter

{
	static const MFDBUTTONMENU mnu[11] = {
		{"Toggle Help","On/Off",'H'},
		{"Step Forward","/ Add Step",'F'},
		{"Step Back",0,'R'},
		{"Select View",0,'W'},
		{"Next Variable", 0, 'V'},
		{"Prev. Variable", 0, 'B'},
		{"Set adjustment", "method (+)", '{'},
		{"Set adjustment", "method (-)", '}'},
		{"Increment", "variable", '='},
		{"Decrement", "variable", '-'},
		{"Switch from","All/Variables",'X'}
	};
	if (menu) *menu = mnu;
	return 10;
}

bool TransxMFD::ConsumeKeyImmediate(char *kstate)

// Routine which processes the keyboard state. Allows continuous change of variable.

{
	if (!valid) return false;
	MFDvariable *varpointer=viewstate->GetCurrVariable();
	if (varpointer==NULL) return false;
	if  (*(kstate+OAPI_KEY_MINUS)==-128)
	{
		return varpointer->dec_variable();
	}
	if (*(kstate+OAPI_KEY_EQUALS)==-128)
	{
		return varpointer->inc_variable();
	}
	return false;
}



bool TransxMFD::ConsumeButton(int bt, int event)

// Deal with mouse pressing of keys

{
	static const DWORD btkey[11]={OAPI_KEY_H, OAPI_KEY_F, OAPI_KEY_R, OAPI_KEY_W, OAPI_KEY_PERIOD, 
		OAPI_KEY_COMMA, OAPI_KEY_LBRACKET, OAPI_KEY_RBRACKET, OAPI_KEY_MINUS, OAPI_KEY_EQUALS, OAPI_KEY_X};
	if (!valid) return false;
	MFDvariable *varpointer=viewstate->GetCurrVariable();
	if ((event & PANEL_MOUSE_LBDOWN) && (bt<8 || bt==10)) return ConsumeKeyBuffered (btkey[bt]);
	if (bt>9) return false; //No buttons above 10
	if (varpointer==NULL) return false;
	if ((event & PANEL_MOUSE_LBDOWN) && varpointer->getadjtype()==0)
	{
		// Button 8 or 9 is pressed and the mode is discrete
		return ConsumeKeyBuffered (btkey[bt]);
	}
	// At this point, only the possibility of immediate consumption...
	if (!(event & PANEL_MOUSE_LBPRESSED)) return false; //Mouse button is down on 9 or 10
	if (bt==9)
	{
		varpointer->dec_variable();
	}
	if (bt==8)
	{
		varpointer->inc_variable();
	}
	return true;
}


bool TransxMFD::ConsumeKeyBuffered (DWORD key)

{
	if (!valid) return false;
	MFDvariable *currvar=viewstate->GetCurrVariable();
	bool access=(currvar!=NULL);
	switch (key) {
	case OAPI_KEY_H:
		viewstate->fliphelpsystem();
		return true;
	case OAPI_KEY_F:
		viewstate->movetonextfunction();
		return true;
	case OAPI_KEY_R:
		viewstate->movetopreviousfunction();
		return true;
	case OAPI_KEY_W:
		viewstate->inc_viewmode();
		return true;
	case OAPI_KEY_X:
		viewstate->togglefunctionswitch();
		return true;
	case OAPI_KEY_PERIOD: //Switch variable
		{
			int viewmode=viewstate->getvariableviewmode();
			viewstate->GetVarhandler()->setnextcurrent(viewmode);
		}
		return true;
	case OAPI_KEY_COMMA:
		{
			int viewmode=viewstate->getvariableviewmode();
			viewstate->GetVarhandler()->setprevcurrent(viewmode);
		}
		return true;
	case OAPI_KEY_LBRACKET:
		if (access) currvar->ch_adjmode();
		return true;
	case OAPI_KEY_RBRACKET:
		if (access) currvar->chm_adjmode();
		return true;
	case OAPI_KEY_MINUS:
		if (access)	return currvar->flipback_variable();
		return true;
	case OAPI_KEY_EQUALS:
		if (access) return currvar->flip_variable();
		return true;
	}
	return false; //Key not one of cases above
}


/*bool TransxMFD::SetResetflag(char *str)
{
	if (!valid) return false;
	if (str[0]=='y' || str[0]=='Y') 
	{
		state->GetCurrFunction()->initialisevars();
	}
	return true;
}*/



void TransxMFD::WriteStatus(FILEHANDLE scn) const
{
	if (!valid) return;
	viewstate->savecurrent(scn);
}


void TransxMFD::ReadStatus(FILEHANDLE scn)
{
	if (viewstate==NULL) return;
	viewstate->restoresave(scn);
}
